"use strict";

exports.__esModule = true;
exports.min = min;
exports.max = max;
exports.expand = expand;
exports.cross = cross;
exports.applyMatrix = applyMatrix;
exports.distance = distance;
exports.unpack = unpack;

var _inflate = require("pako/lib/inflate");

var _base64Arraybuffer = require("base64-arraybuffer");

function min(vec, v) {
  vec[0] = Math.min(vec[0], v[0]);
  vec[1] = Math.min(vec[1], v[1]);
  vec[2] = Math.min(vec[2], v[2]);
}

function max(vec, v) {
  vec[0] = Math.max(vec[0], v[0]);
  vec[1] = Math.max(vec[1], v[1]);
  vec[2] = Math.max(vec[2], v[2]);
}

function expand(vecMin, vecMax, v) {
  min(vecMin, v);
  max(vecMax, v);
}

function cross(vec, v) {
  var x = vec[0],
      y = vec[1],
      z = vec[2];
  vec[0] = y * v[2] - z * v[1];
  vec[1] = z * v[0] - x * v[2];
  vec[2] = x * v[1] - y * v[0];
}

function applyMatrix(vec, e) {
  var x = vec[0],
      y = vec[1],
      z = vec[2];
  vec[0] = e[0] * x + e[4] * y + e[8] * z + e[12];
  vec[1] = e[1] * x + e[5] * y + e[9] * z + e[13];
  vec[2] = e[2] * x + e[6] * y + e[10] * z + e[14];
}

function distance(v1, v2) {
  var x1 = v1[0],
      y1 = v1[1],
      z1 = v1[2];
  var x2 = v2[0],
      y2 = v2[1],
      z2 = v2[2];
  var dx = x1 - x2,
      dy = y1 - y2,
      dz = z1 - z2;
  return Math.sqrt(dx * dx + dy * dy + dz * dz);
}

function unpack(blob) {
  var stream = blob.data;
  if (!(stream instanceof Uint8Array)) stream = (0, _base64Arraybuffer.decode)(blob.data);
  blob.data = JSON.parse((0, _inflate.inflate)(stream, {
    raw: true,
    to: 'string'
  }));
  blob.size = stream.length;
}